import click

from datetime import timezone


class Ternary:
    true_val = 1
    false_val = 0
    clear_val = 2

    def __init__(self, value):
        self.value = value

    def is_true(self):
        return self.value == self.true_val

    def is_false(self):
        return self.value == self.false_val

    def is_clear(self):
        return self.value == self.clear_val

    def to_bool_or_none(self):
        if self.is_true():
            return True
        if self.is_false():
            return False
        if self.is_clear():
            return None


class TernaryParamType(click.types.BoolParamType):
    name = "ternary"
    CLEAR = "clear"
    CHOICES = "BOOL|clear"

    def convert(self, value, param, ctx):
        if value == self.CLEAR:
            return Ternary(Ternary.clear_val)
        try:
            if super().convert(value, param, ctx):
                return Ternary(Ternary.true_val)
            return Ternary(Ternary.false_val)
        except click.BadParameter:
            pass

        self.fail(f"{value!r} is not a valid bool or 'clear'")

    def to_info_dict(self):
        info_dict = super().to_info_dict()
        info_dict["choices"] = self.CHOICES
        return info_dict

    def get_metavar(self, param, ctx) -> str:
        return self.CHOICES


class DateTime(click.DateTime):
    def __init__(self, formats=None):
        super().__init__(formats)
        self.formats = list(self.formats)
        self.formats.append("%Y-%m-%dT%H:%M:%S%z")

    def convert(self, value, param, ctx):
        converted = super().convert(value, param, ctx)
        if converted is not None:
            return converted.astimezone(timezone.utc)
        return None



from typing import List
import sys
import socket

import agilicus
import json

from . import context
from .input_helpers import strip_none
from .input_helpers import add_remove_uniq_list
from .output.table import (
    spec_column,
    format_table,
    metadata_column,
    status_column,
)


def list_point_of_presences(
    ctx, excludes_all_tag, excludes_any_tag, includes_all_tag, includes_any_tag, **kwargs
):
    apiclient = context.get_apiclient_from_ctx(ctx)
    excludes_all_tags = tag_list_to_tag_names(excludes_all_tag)
    excludes_any_tags = tag_list_to_tag_names(excludes_any_tag)
    includes_all_tags = tag_list_to_tag_names(includes_all_tag)
    includes_any_tags = tag_list_to_tag_names(includes_any_tag)

    return apiclient.regions_api.list_point_of_presences(
        excludes_all_tag=excludes_all_tags,
        excludes_any_tag=excludes_any_tags,
        includes_all_tag=includes_all_tags,
        includes_any_tag=includes_any_tags,
        **strip_none(kwargs),
    ).point_of_presences


def add_point_of_presence(ctx, name, tag: List[str], domain=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    tags = []
    if tag:
        tags = tag_list_to_tag_names(tag)

    domains = []
    if domain:
        domains = [agilicus.Domain(d) for d in domain]

    routing = agilicus.PointOfPresenceRouting(domains=domains)
    pop_spec = agilicus.PointOfPresenceSpec(
        name=agilicus.FeatureTagName(name), tags=tags, routing=routing
    )
    pop = agilicus.PointOfPresence(spec=pop_spec)
    return apiclient.regions_api.add_point_of_presence(pop)


def update_point_of_presence(
    ctx,
    pop_id,
    tag: List[str],
    domain=None,
    overwrite_tags=False,
    overwrite_domains=False,
    name=None,
    add_cluster_ids=None,
    remove_cluster_ids=None,
    master_cluster_id=None,
    requests_enabled=None,
    org_domain=None,
    overwrite_org_domains=False,
    public=None,
    restrict_by_user_id=None,
    add_permitted_user_id=None,
    remove_permitted_user_id=None,
    routing_ces=None,
    **kwargs,
):
    apiclient = context.get_apiclient_from_ctx(ctx)

    original = apiclient.regions_api.get_point_of_presence(point_of_presence_id=pop_id)

    tags = []
    if tag:
        tags = tag_list_to_tag_names(tag)

    if not overwrite_tags:
        tags.extend(original.spec.tags)
        tags = tag_list_to_tag_names(list(set(str(tag) for tag in tags)))

    original.spec.tags = tags

    domains = []
    if domain:
        domains = [agilicus.Domain(d) for d in domain]

    org_domains = []
    if org_domain:
        org_domains = [agilicus.Domain(d) for d in org_domain]

    if not overwrite_domains:
        to_write = original.spec.routing.domains
        for domain in domains:
            if domain not in to_write:
                to_write.append(domain)
        domains = to_write

    if not overwrite_org_domains:
        to_write = original.spec.routing.org_domains
        for domain in org_domains:
            if domain not in to_write:
                to_write.append(domain)
        org_domains = to_write

    original.spec.routing.domains = domains
    original.spec.routing.org_domains = org_domains
    if name is not None:
        original.spec.name = name

    if requests_enabled is not None:
        original.spec.routing.requests_enabled = requests_enabled

    if public is not None:
        original.spec.routing.public = public

    if restrict_by_user_id is not None:
        original.spec.routing.restrict_by_user_id = restrict_by_user_id

    if routing_ces is not None:
        original.spec.routing.ces = routing_ces

    original.spec.routing.permitted_user_ids = add_remove_uniq_list(
        original.spec.routing.permitted_user_ids,
        add_permitted_user_id,
        remove_permitted_user_id,
    )

    original.spec.cluster_ids = add_remove_uniq_list(
        original.spec.cluster_ids,
        add_cluster_ids,
        remove_cluster_ids,
    )
    if master_cluster_id is not None:
        original.spec.master_cluster_id = master_cluster_id

    return apiclient.regions_api.replace_point_of_presence(
        pop_id,
        point_of_presence=original,
    )


def show_point_of_presence(ctx, pop_id):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.get_point_of_presence(point_of_presence_id=pop_id)


def delete_point_of_presence(ctx, pop_id):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.delete_point_of_presence(point_of_presence_id=pop_id)


def format_point_of_presences_as_text(ctx, tags):
    columns = [
        metadata_column("id"),
        spec_column("name"),
        spec_column("tags"),
        spec_column("routing", "routing"),
        spec_column("master_cluster_id"),
        status_column("clusters"),
    ]

    return format_table(ctx, tags, columns)


def tag_list_to_tag_names(tags: List[str]) -> List[agilicus.FeatureTagName]:
    return [agilicus.FeatureTagName(tag_name) for tag_name in tags]


def add_cluster(ctx, name, ip_addresses=None, description=None, domain=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    config = agilicus.ClusterConfig()

    if ip_addresses:
        config.ip_addresses = ip_addresses

    if description:
        config.description = description

    spec = agilicus.ClusterSpec(
        name=agilicus.Domain(name),
        config=config,
    )

    if domain:
        spec.domain = agilicus.Domain(domain)

    cluster = agilicus.Cluster(spec=spec)
    return apiclient.regions_api.add_cluster(cluster)


def _update_list(ip_addresses):
    return {k: True for v, k in enumerate(ip_addresses)}


def update_cluster(
    ctx,
    cluster_id,
    name=None,
    domain=None,
    remove_ip_addresses=None,
    add_ip_addresses=None,
    description=None,
    **kwargs,
):
    apiclient = context.get_apiclient_from_ctx(ctx)
    cluster = apiclient.regions_api.get_cluster(cluster_id)

    config = cluster.spec.config
    if not config:
        cluster.spec.config = agilicus.ClusterConfig()
        config = cluster.spec.config

    config.ip_addresses = add_remove_uniq_list(
        config.ip_addresses,
        add_ip_addresses,
        remove_ip_addresses,
    )

    if description:
        config.description = description

    if name:
        cluster.spec.name = agilicus.Domain(name)

    if domain:
        cluster.spec.domain = agilicus.Domain(domain)

    return apiclient.regions_api.replace_cluster(cluster_id, cluster=cluster)


def delete_cluster(ctx, cluster_id):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.delete_cluster(cluster_id)


def list_clusters(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    return apiclient.regions_api.list_clusters(
        **strip_none(kwargs),
    ).clusters


def list_regions(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    return apiclient.regions_api.list_regions(
        **strip_none(kwargs),
    ).regions


def add_region(ctx, name, domain=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    domains = []
    if domain:
        domains = [agilicus.Domain(d) for d in domain]

    routing = agilicus.RegionRouting(domains=domains)
    region_spec = agilicus.RegionSpec(name=name, routing=routing)
    region = agilicus.Region(spec=region_spec)
    return apiclient.regions_api.add_region(region)


def update_region(
    ctx,
    region_id,
    domain=None,
    overwrite_domains=False,
    name=None,
    add_pop_ids=None,
    remove_pop_ids=None,
    master_pop_id=None,
    requests_enabled=None,
    org_domain=None,
    public=None,
    restrict_by_user_id=None,
    add_permitted_user_id=None,
    remove_permitted_user_id=None,
    routing_ces=None,
    **kwargs,
):
    apiclient = context.get_apiclient_from_ctx(ctx)

    original = apiclient.regions_api.get_region(region_id=region_id)

    domains = []
    if domain:
        domains = [agilicus.Domain(d) for d in domain]

    org_domains = []
    if org_domain:
        org_domains = [agilicus.Domain(d) for d in org_domain]

    if not overwrite_domains:
        to_write = original.spec.routing.domains
        for domain in domains:
            if domain not in to_write:
                to_write.append(domain)
        domains = to_write

    original.spec.routing.domains = domains
    original.spec.routing.org_domains = org_domains
    if name is not None:
        original.spec.name = name

    if requests_enabled is not None:
        original.spec.routing.requests_enabled = requests_enabled

    if public is not None:
        original.spec.routing.public = public

    if restrict_by_user_id is not None:
        original.spec.routing.restrict_by_user_id = restrict_by_user_id

    if routing_ces is not None:
        original.spec.routing.ces = routing_ces

    original.spec.routing.permitted_user_ids = add_remove_uniq_list(
        original.spec.routing.permitted_user_ids,
        add_permitted_user_id,
        remove_permitted_user_id,
    )

    original.spec.pop_ids = add_remove_uniq_list(
        original.spec.pop_ids,
        add_pop_ids,
        remove_pop_ids,
    )
    if master_pop_id is not None:
        original.spec.master_pop_id = master_pop_id

    return apiclient.regions_api.replace_region(
        region_id,
        region=original,
    )


def delete_region(ctx, region_id):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.delete_region(region_id=region_id)


def _filter_requests_enabled(objs):
    filtered = []
    for obj in objs:
        if obj.spec.routing and obj.spec.routing.public:
            filtered.append(obj)
    return filtered


def _add_rule(name, domains, ip_addresses, ports):
    return {
        "name": name,
        "domains": domains,
        "ip_addresses": ip_addresses,
        "ports": ports,
    }


def _get_ip_addresses(obj):
    ip_addresses = []
    clusters = []
    if obj.status.get("pops"):
        for pop in obj.status.pops:
            clusters = pop.status.clusters or []
    elif obj.status.get("clusters"):
        clusters = obj.status.clusters

    for cluster in clusters:
        ip_addresses.extend(cluster.spec.config.ip_addresses or [])
    return set(ip_addresses)


def _get_ip_addresses_by_name(
    domains, port, obj, fail_on_domain_lookup_failure=None, **kwargs
):
    ip_addresses = set()
    for domain in domains:
        try:
            for addr in socket.getaddrinfo(domain, port):
                ip_addresses.add(addr[4][0])
        except Exception:
            print(f"no lookup for domain {domain}", file=sys.stderr)
            if fail_on_domain_lookup_failure:
                sys.exit(1)

    if obj:
        ip_addresses = ip_addresses.union(_get_ip_addresses(obj))

    return list(ip_addresses)


def _add_location(location_type, obj, **kwargs):
    loc = {}
    loc["name"] = str(obj.spec.name)
    loc["location_type"] = location_type
    hosting = loc.setdefault("hosting", {})
    hosting["cname_domain_forwards"] = [
        str(domain) for domain in obj.spec.routing.domains or []
    ]
    hosting["org_domains"] = [
        str(domain) for domain in obj.spec.routing.org_domains or []
    ]
    rules = hosting.setdefault("firewall-rules", {})
    allow = rules.setdefault("allow", [])
    allow.append(
        _add_rule(
            "www",
            ["www.agilicus.com"],
            _get_ip_addresses_by_name(["www.agilicus.com"], 443, None, **kwargs),
            [443],
        )
    )
    allow.append(
        _add_rule(
            "noc",
            ["api.agilicus.com"],
            _get_ip_addresses_by_name(["api.agilicus.com"], 443, None, **kwargs),
            [443],
        )
    )

    domains = []
    for domain in obj.spec.routing.domains or []:
        domains.append(str(domain))
        domains.append(f"agent-server.{domain}")
        domains.append(f"desktops.{domain}")
        domains.append(f"ssh-gateway.{domain}")
    allow.append(
        _add_rule(
            "admin/dataplane",
            domains,
            _get_ip_addresses_by_name(domains, 443, obj, **kwargs),
            [443],
        )
    )
    return loc


def build_agilicus_locations(ctx, *args, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)

    regions = _filter_requests_enabled(apiclient.regions_api.list_regions().regions)
    pops = _filter_requests_enabled(
        apiclient.regions_api.list_point_of_presences().point_of_presences
    )

    top = {}
    locations = top.setdefault("agilicus_locations", [])
    for region in regions:
        locations.append(_add_location("region", region, **kwargs))

    for pop in pops:
        locations.append(_add_location("pop", pop, **kwargs))
    print(json.dumps(top))


def routing_request(ctx, *args, ip_address=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    ip_addresses = []
    if ip_address is not None:
        ip_addresses = list(ip_address)
    request = agilicus.RoutingRequest(ip_addresses=ip_addresses)
    return apiclient.regions_api.routing_request(request)


def list_regional_locations(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return apiclient.regions_api.get_regional_locations(**strip_none(kwargs))



from .. import context
import yaml
import agilicus
from ..input_helpers import update_org_from_input_or_ctx, pop_item_if_none
from .. import create_or_update


def list_deployments(ctx, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.list_deployments(**kwargs).deployments


def get_deployment(ctx, deployment_id, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.get_deployment(deployment_id, **kwargs)


def delete_deployment(ctx, deployment_id, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.delete_deployment(deployment_id, **kwargs)


def add_deployment(ctx, schema=None, **kwargs):
    config_data = {}
    if schema:
        with open(schema, "r") as file:
            config_data = yaml.safe_load(file)
    pop_item_if_none(kwargs)
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)

    deployment = agilicus.Deployment(
        agilicus.DeploymentSpec(**kwargs, schema=config_data)
    )
    return apiclient.deployments_api.create_deployment(deployment)


def list_deployment_instances(ctx, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.list_deployment_instances(
        **kwargs
    ).deployment_instances


def get_deployment_instance(ctx, deployment_instance_id, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.get_deployment_instance(
        deployment_instance_id, **kwargs
    )


def add_deployment_instance(ctx, input, **kwargs):
    pop_item_if_none(kwargs)
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)

    inputs = []
    for _input in input:
        split_str = _input.split(",")
        name = split_str[0]
        value = split_str[1]
        inputs.append(agilicus.DeploymentInstanceInput(name=name, value_as_string=value))
    instance = agilicus.DeploymentInstance(
        agilicus.DeploymentInstanceSpec(inputs=inputs, **kwargs)
    )
    return apiclient.deployments_api.create_deployment_instance(instance)


def delete_deployment_instance(ctx, deployment_instance_id, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.delete_deployment_instance(
        deployment_instance_id, **kwargs
    )


def list_deployment_templates(ctx, **kwargs):
    pop_item_if_none(kwargs)
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.list_deployment_templates(
        **kwargs
    ).deployment_templates


def get_deployment_template(ctx, deployment_template_id, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.get_deployment_template(
        deployment_template_id, **kwargs
    )


def add_deployment_template(ctx, template, **kwargs):
    with open(template, "r") as file:
        config_data = yaml.safe_load(file)
        return add_deployment_template_from_dict(ctx, config_data, **kwargs)


def add_deployment_template_from_dict(ctx, template, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["description"] = template.get("description")
    pop_item_if_none(kwargs)

    deployment_template = agilicus.DeploymentTemplate(
        agilicus.DeploymentTemplateSpec(**kwargs, template=template)
    )

    return create_or_update(
        deployment_template,
        apiclient.deployments_api.create_deployment_template,
        apiclient.deployments_api.update_deployment_template,
    )


def update_system_templates(ctx, templates, **kwargs):
    with open(templates, "r") as file:
        config_data = yaml.safe_load(file)
        for template_type, v in config_data.items():
            for name, template_dict in v.items():
                add_deployment_template_from_dict(
                    ctx,
                    name=name,
                    template=template_dict,
                    template_type=template_type,
                    **kwargs,
                )


def delete_deployment_template(ctx, deployment_template_id, **kwargs):
    token = context.get_token(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    apiclient = context.get_apiclient(ctx, token)
    return apiclient.deployments_api.delete_deployment_template(
        deployment_template_id, **kwargs
    )



"""
AutoIterator for paginating API client list functions.
"""

import json
from agilicus_api.exceptions import ApiException

_page_on_preference = ["name", "created", "id"]
_default_page_on = "id"


def get_page_on(list_function):
    """
    tries to figure out the pagination key to use based on the endpoint:
     - If it's an enum, uses the preferred one in the list, otherwise the first
     - If it's not an enum, just tries the 'default', which is id.
    """
    # yes, the key here is a tuple or some reason
    result = list(list_function.allowed_values.get(("page_on",), {}).values())
    if not result and "page_on" in list_function.params_map["all"]:
        return [_default_page_on]
    return result


def _determine_page_on(list_function):
    page_on_list = get_page_on(list_function)
    if not page_on_list:
        raise NotImplementedError("pagination not implemented on this endpoint")
    for page_on in _page_on_preference:
        if page_on in page_on_list:
            return [page_on]

    return [page_on_list[0]]


def patch_endpoint(list_function):
    setattr(list_function, "auto_paging_iter", AutoIteratorWrapper(list_function))


class AutoIteratorWrapper:
    def __init__(self, list_function):
        self.list_function = list_function

    def __call__(self, *args, limit=None, page_size=100, page_sort=None, **kwargs):
        return AutoIterator(
            self.list_function, limit, page_size, page_sort, *args, **kwargs
        )


class AutoIterator:
    PLACE_HOLDER = object()

    def __init__(
        self,
        list_function,
        limit,
        page_size,
        page_sort,
        *args,
        page_on=None,
        legacy_pagination=False,
        **kwargs,
    ):
        """
        Initializes the AutoIterator.

        :param list_function: The API client list function to paginate.
        :param args: Positional arguments to pass to the list_function.
        :param kwargs: Keyword arguments to pass to the list_function.
        """
        self._list_function = list_function
        self._args = args
        self._kwargs = kwargs
        self.page_sort = page_sort or []
        self.limit = limit
        self.page_size = page_size
        self.legacy_pagination = legacy_pagination
        if not self.legacy_pagination:
            self.configured_page_on = page_on or _determine_page_on(self._list_function)
        else:
            self.configured_page_on = page_on or "page_at_id"
        self._reset()

    def _reset(self):
        self._total_items = 0
        self._page_on = self.configured_page_on
        if self.legacy_pagination:
            self._next_page = ""
        else:
            self._next_page = []
        self._has_more = True
        self._current_page_items = []

    def __iter__(self):
        """
        Returns the iterator object itself.
        """
        return self

    def __next__(self):
        """
        Fetches the next item from the paginated results.
        """
        if not self._current_page_items and self._has_more:
            self._fetch_next_page()

        if self._current_page_items:
            return self._current_page_items.pop(0)
        else:
            raise StopIteration

    def _get_page_list(self, response):
        for key in response.attribute_map:
            if isinstance(response.get(key), list):
                return response.get(key)
        return []

    def _build_page_params(self):
        params = {}
        if self.legacy_pagination:
            params[self.configured_page_on] = self._next_page
        else:
            next_page_on = self._next_page
            if not isinstance(next_page_on, list) and hasattr(next_page_on, "value"):
                # Convert to a list
                next_page_on = next_page_on.value
            params["page_on"] = self._page_on
            params["page_at_key"] = next_page_on
            params["page_sort"] = self.page_sort
        return params

    def _fetch_next_page(self):
        limit = self.page_size
        if self.limit is not None:
            limit = min(limit, self.limit - self._total_items)
        page_params = self._build_page_params()
        params = self._kwargs | page_params
        try:
            response = self._list_function(
                limit=limit,
                **params,
            )
        except ApiException as exc:
            if exc.status != 400:
                raise

            _handle_page_failure(exc.body)
            # Shut up the linter, which thinks the above doesn't always throw even
            # though it does
            assert False

        self._current_page_items = self._get_page_list(response)
        self._total_items = self._total_items + len(self._current_page_items)
        if not self._build_next_page(response):
            self._has_more = False
            return

    def _build_next_page(self, response):
        if not self._current_page_items or (
            self.limit and self._total_items >= self.limit
        ):
            return False

        if self.legacy_pagination:
            if len(self._current_page_items) < self.page_size:
                return False
            next_page = response.get(self.configured_page_on, self.PLACE_HOLDER)
            if next_page is self.PLACE_HOLDER:
                return False
            self._next_page = next_page
            return True

        page_info = response.page_info
        if not page_info:
            return False

        self._next_page = page_info.page_at_key
        self._page_on = page_info.page_on.value
        return True

    # Additional functionality when interacted with as an object
    def all(self):
        """
        Returns all items by iterating through all pages.
        """
        all_items = []
        for item in self:
            all_items.append(item)
        return all_items

    def first(self):
        """
        Returns the first item, or None if no items.
        """
        self._reset()
        try:
            return next(self)
        except StopIteration:
            return None

    def __repr__(self):
        return f"<AutoIterator for {self._list_function.__name__}>"


def _handle_page_failure(body):
    if not body:
        raise
    result = {}
    try:
        result = json.loads(body)
    except Exception:
        pass
    error_message = result.get("error_message", "")
    if "pagination key" not in error_message:
        raise

    raise NotImplementedError("Pagination not implemented on this key") from None



import regex

from abc import abstractmethod
from typing import Optional

from agilicus import (
    RuleAction,
    RuleCondition,
    RuleConfig,
    MaxMFAProofRuleCondition,
    StandaloneObjectConditions,
    StandaloneRule,
    StandaloneRuleName,
    StandaloneRuleScope,
    StandaloneRuleSpec,
    StandaloneRuleTree,
    StandaloneRuleTreeSpec,
    StandaloneRuleTreeRef,
    StandaloneRuleTreeRuleRef,
    StandaloneRuleTreeNode,
    StandaloneRuleset,
    StandaloneRulesetLabelName,
    StandaloneRulesetLabelSpec,
    StandaloneRulesetLabel,
    StandaloneRulesetSpec,
)


class PolicyTemplate:
    @abstractmethod
    def get_rules(self) -> list[StandaloneRule]:
        pass

    @abstractmethod
    def get_trees(self) -> list[StandaloneRuleTree]:
        pass

    @abstractmethod
    def get_sets(self) -> list[StandaloneRuleset]:
        pass

    @abstractmethod
    def get_labels(self) -> list[StandaloneRulesetLabel]:
        pass


def mfa_cond(max_seconds):
    return MaxMFAProofRuleCondition(
        condition_type="mfa_rule_condition", max_seconds=max_seconds
    )


MFA_LABEL = "global-multifactor"
LABEL_REGEX = regex.compile(r"urn:agilicus:label:(?P<label>[a-zA-Z0-9\-\._:]+)")


def parse_label(scope):
    match = LABEL_REGEX.match(str(scope))
    if not match:
        return None
    return match.group("label")


def parse_labels(scopes):
    result = []
    for scope in scopes:
        label = parse_label(scope)
        if not label:
            continue
        result.append(label)
    return result


class MultifactorTemplate(PolicyTemplate):
    name_regex = regex.compile(r"agilicus-mfa-tmpl-(?P<name>[a-zA-Z0-9\-\._]+)")

    def __init__(self, name, duration, org_id, labels: Optional[list[str]] = None):
        self.name = name
        self.duration = duration
        self.labels = labels or []
        self.org_id = org_id

    def to_dict(self):
        return {
            "name": self.name,
            "duration": self.duration,
            "labels": self.labels,
            "org_id": self.org_id,
        }

    @classmethod
    def build_name(cls, name):
        return f"agilicus-mfa-tmpl-{name}"

    def get_name(self):
        return self.build_name(self.name)

    @classmethod
    def parse_name(cls, name):
        match = cls.name_regex.match(name)
        if not match:
            return None
        return match.group("name")

    def get_rules(self) -> list[StandaloneRule]:
        cond = mfa_cond(self.duration)
        action = RuleAction(action="mfa")
        extended_condition = RuleCondition(negated=True, condition=cond)
        spec = StandaloneRuleSpec(
            rule=RuleConfig(
                name=self.get_name(),
                extended_condition=extended_condition,
                actions=[action],
            ),
            org_id=self.org_id,
        )

        rule = StandaloneRule(spec=spec)
        return [rule]

    def get_trees(self) -> list[StandaloneRuleTree]:
        spec = StandaloneRuleTreeSpec(
            tree=StandaloneRuleTreeNode(
                rules=[
                    StandaloneRuleTreeRuleRef(
                        rule_name=StandaloneRuleName(self.get_name()),
                    )
                ],
                children=[],
            ),
            name=StandaloneRuleName(self.get_name()),
            org_id=self.org_id,
            object_conditions=StandaloneObjectConditions(),
            # object_conditions=StandaloneObjectConditions(
            #     scopes=[StandaloneRuleScope("urn:agilicus:scope:any_resource_user")]
            # )
        )

        return [StandaloneRuleTree(spec=spec)]

    def get_sets(self) -> list[StandaloneRuleset]:
        spec = StandaloneRulesetSpec(
            rule_trees=[
                StandaloneRuleTreeRef(
                    rule_tree_name=StandaloneRuleName(self.get_name()),
                    priority=0,
                )
            ],
            labels=[StandaloneRulesetLabelName(MFA_LABEL)],
            name=StandaloneRuleName(self.get_name()),
            org_id=self.org_id,
        )
        scopes = [
            StandaloneRuleScope(f"urn:agilicus:label:{label}") for label in self.labels
        ]
        if scopes:
            object_conditions = StandaloneObjectConditions(scopes=scopes)
            spec.object_conditions = object_conditions

        return [StandaloneRuleset(spec=spec)]

    def get_labels(self) -> list[StandaloneRulesetLabel]:
        spec = StandaloneRulesetLabelSpec(
            name=StandaloneRulesetLabelName(MFA_LABEL),
            org_id=self.org_id,
        )

        return [
            StandaloneRulesetLabel(spec=spec),
        ]

    @classmethod
    def from_api(cls, name, rules, org_id, trees, sets):
        found_sets = list(filter(lambda s: str(s.spec.name) == name, sets))
        found_trees = list(filter(lambda s: str(s.spec.name) == name, trees))
        found_rules = list(filter(lambda s: str(s.spec.rule.name) == name, rules))

        if len(found_sets) == 0 or len(found_rules) == 0 or len(found_trees) == 0:
            return None

        # There should be only one, given that names are unique.
        ruleset = found_sets[0]
        labels = []
        if ruleset.spec.object_conditions:
            labels = parse_labels(ruleset.spec.object_conditions.scopes)
        rule = found_rules[0]
        try:
            duration = rule.spec.rule.extended_condition.condition.max_seconds
        except Exception:
            return None

        return MultifactorTemplate(
            name=cls.parse_name(name), duration=duration, labels=labels, org_id=org_id
        )



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2026.03.25
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""



class OpenApiException(Exception):
    """The base exception class for all OpenAPIExceptions"""


class ApiTypeError(OpenApiException, TypeError):
    def __init__(self, msg, path_to_item=None, valid_classes=None,
                 key_type=None):
        """ Raises an exception for TypeErrors

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list): a list of keys an indices to get to the
                                 current_item
                                 None if unset
            valid_classes (tuple): the primitive classes that current item
                                   should be an instance of
                                   None if unset
            key_type (bool): False if our value is a value in a dict
                             True if it is a key in a dict
                             False if our item is an item in a list
                             None if unset
        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiTypeError, self).__init__(full_msg)


class ApiValueError(OpenApiException, ValueError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list) the path to the exception in the
                received_data dict. None if unset
        """

        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiValueError, self).__init__(full_msg)


class ApiAttributeError(OpenApiException, AttributeError):
    def __init__(self, msg, path_to_item=None):
        """
        Raised when an attribute reference or assignment fails.

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiAttributeError, self).__init__(full_msg)


class ApiKeyError(OpenApiException, KeyError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiKeyError, self).__init__(full_msg)


class ApiException(OpenApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n"\
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class NotFoundException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(NotFoundException, self).__init__(status, reason, http_resp)


class UnauthorizedException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(UnauthorizedException, self).__init__(status, reason, http_resp)


class ForbiddenException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(ForbiddenException, self).__init__(status, reason, http_resp)


class ServiceException(ApiException):

    def __init__(self, status=None, reason=None, http_resp=None):
        super(ServiceException, self).__init__(status, reason, http_resp)


def render_path(path_to_item):
    """Returns a string representation of a path"""
    result = ""
    for pth in path_to_item:
        if isinstance(pth, int):
            result += "[{0}]".format(pth)
        else:
            result += "['{0}']".format(pth)
    return result



from .. import context
from ..input_helpers import (
    get_org_from_input_or_ctx,
    strip_none,
)
import agilicus
from ..output.table import (
    spec_column,
    format_table,
    metadata_column,
)


def add_public_file_org_link(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs = strip_none(kwargs)

    spec = agilicus.PublicFileOrgLinkSpec(**kwargs)

    obj = agilicus.PublicFileOrgLink(spec=spec)
    return apiclient.files_api.create_public_file_org_link(obj).to_dict()


def list_public_file_org_links(ctx, link_org_id=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["link_org_id"] = get_org_from_input_or_ctx(ctx, org_id=link_org_id)
    kwargs = strip_none(kwargs)

    return apiclient.files_api.list_public_file_org_links(**kwargs).public_file_org_links


def get_public_file_org_link(ctx, link_org_id=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["link_org_id"] = get_org_from_input_or_ctx(ctx, org_id=link_org_id)
    kwargs = strip_none(kwargs)

    return apiclient.files_api.get_public_file_org_link(**kwargs)


def delete_public_file_org_link(ctx, link_org_id=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["link_org_id"] = get_org_from_input_or_ctx(ctx, org_id=link_org_id)
    kwargs = strip_none(kwargs)

    apiclient.files_api.delete_public_file_org_link(**kwargs)


def format_public_file_org_links(ctx, labels):
    columns = [
        metadata_column("id"),
        spec_column("link_org_id"),
        spec_column("target_org_id"),
        spec_column("file_tag"),
    ]

    return format_table(ctx, labels, columns)



import click
from . import credentials
from ..output.table import output_entry


@click.command(name="add-object-credential")
@click.option("--object-id", required=True)
@click.option("--object-type", required=True)
@click.option("--org-id", default=None)
@click.option("--priority", type=int, default=0)
@click.option("--purpose", required=True)
@click.option("--encrypt", default=True)
@click.option("--username", default=None)
@click.option("--password", default=None)
@click.option("--private-key", type=click.File("r"), default=None)
@click.option("--private-key-passphrase", default=None)
@click.option("--oauth2-token-url", default=None, type=str)
@click.option("--oauth2-client-id", default=None, type=str)
@click.option("--oauth2-scopes", default=None, multiple=True, type=str)
@click.option("--oauth2-federated-audience", default=None, type=str)
@click.option("--oauth2-bearer-fingerprint", default=None, type=str)
@click.pass_context
def cli_command_add_object_credential(
    ctx, private_key, username, password, encrypt, private_key_passphrase, **kwargs
):
    secrets = credentials.make_secrets(
        private_key=private_key,
        username=username,
        password=password,
        encrypt=encrypt,
        private_key_passphrase=private_key_passphrase,
    )

    output_entry(ctx, credentials.add_object_credentials(ctx, secrets=secrets, **kwargs))


@click.command(name="replace-object-credential")
@click.option("--object-credential-id", required=True)
@click.option("--oauth2-token-url", default=None, type=str)
@click.option("--oauth2-client-id", default=None, type=str)
@click.option("--oauth2-scopes", default=None, multiple=True, type=str)
@click.option("--oauth2-federated-audience", default=None, type=str)
@click.option("--oauth2-bearer-fingerprint", default=None, type=str)
@click.option("--object-id")
@click.option("--object-type")
@click.option("--org-id", default=None)
@click.option("--priority", type=int, default=0)
@click.option("--purpose")
@click.pass_context
def cli_command_replace_object_credential(ctx, **kwargs):
    output_entry(ctx, credentials.replace_object_credentials(ctx, **kwargs))


@click.command(name="get-object-credential")
@click.option("--object-credential-id", required=True)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_get_object_credential(ctx, **kwargs):
    output_entry(ctx, credentials.get_object_credential(ctx, **kwargs).to_dict())


@click.command(name="delete-object-credential")
@click.option("--object-credential-id", required=True)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_object_credential(ctx, **kwargs):
    credentials.delete_object_credential(ctx, **kwargs)


@click.command(name="list-object-credentials")
@click.option("--org-id", default=None)
@click.option("--purpose", default=None)
@click.option("--object-type", default=None)
@click.option("--object-types", multiple=True, default=None)
@click.option("--object-id", default=None)
@click.option("--object-ids", multiple=True, default=None)
@click.option("--encryption-key-id", default=None)
@click.option("--purposes", type=str, multiple=True, default=None)
@click.pass_context
def cli_command_list_object_credential(ctx, **kwargs):
    creds = credentials.list_object_credentials(ctx, **kwargs)
    print(credentials.format_object_credentials(ctx, creds))


@click.command(name="list-object-credentials-existence-info")
@click.option("--org-id", default=None)
@click.option("--purpose", default=None)
@click.option("--object-type", default=None)
@click.option("--object-types", multiple=True, default=None)
@click.option("--object-id", default=None)
@click.option("--object-ids", multiple=True, default=None)
@click.pass_context
def cli_command_list_object_credentials_existence_info(ctx, **kwargs):
    creds = credentials.list_existence_info(ctx, **kwargs)
    print(credentials.format_object_credential_existence_info(ctx, creds))


all_funcs = [func for func in dir() if "cli_command_" in func]


def add_commands(cli):
    glob = globals()
    for func in all_funcs:
        cli.add_command(glob[func])



from unittest.mock import MagicMock

class DeploymentsApiMock:

    def __init__(self):
        self.mock_create_deployment = MagicMock()
        self.mock_create_deployment_instance = MagicMock()
        self.mock_create_deployment_template = MagicMock()
        self.mock_delete_deployment = MagicMock()
        self.mock_delete_deployment_instance = MagicMock()
        self.mock_delete_deployment_template = MagicMock()
        self.mock_get_deployment = MagicMock()
        self.mock_get_deployment_instance = MagicMock()
        self.mock_get_deployment_template = MagicMock()
        self.mock_list_deployment_instances = MagicMock()
        self.mock_list_deployment_templates = MagicMock()
        self.mock_list_deployments = MagicMock()
        self.mock_update_deployment = MagicMock()
        self.mock_update_deployment_instance = MagicMock()
        self.mock_update_deployment_template = MagicMock()

    def create_deployment(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.create_deployment with MagicMock.
        """
        return self.mock_create_deployment(self, *args, **kwargs)

    def create_deployment_instance(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.create_deployment_instance with MagicMock.
        """
        return self.mock_create_deployment_instance(self, *args, **kwargs)

    def create_deployment_template(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.create_deployment_template with MagicMock.
        """
        return self.mock_create_deployment_template(self, *args, **kwargs)

    def delete_deployment(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.delete_deployment with MagicMock.
        """
        return self.mock_delete_deployment(self, *args, **kwargs)

    def delete_deployment_instance(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.delete_deployment_instance with MagicMock.
        """
        return self.mock_delete_deployment_instance(self, *args, **kwargs)

    def delete_deployment_template(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.delete_deployment_template with MagicMock.
        """
        return self.mock_delete_deployment_template(self, *args, **kwargs)

    def get_deployment(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.get_deployment with MagicMock.
        """
        return self.mock_get_deployment(self, *args, **kwargs)

    def get_deployment_instance(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.get_deployment_instance with MagicMock.
        """
        return self.mock_get_deployment_instance(self, *args, **kwargs)

    def get_deployment_template(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.get_deployment_template with MagicMock.
        """
        return self.mock_get_deployment_template(self, *args, **kwargs)

    def list_deployment_instances(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.list_deployment_instances with MagicMock.
        """
        return self.mock_list_deployment_instances(self, *args, **kwargs)

    def list_deployment_templates(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.list_deployment_templates with MagicMock.
        """
        return self.mock_list_deployment_templates(self, *args, **kwargs)

    def list_deployments(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.list_deployments with MagicMock.
        """
        return self.mock_list_deployments(self, *args, **kwargs)

    def update_deployment(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.update_deployment with MagicMock.
        """
        return self.mock_update_deployment(self, *args, **kwargs)

    def update_deployment_instance(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.update_deployment_instance with MagicMock.
        """
        return self.mock_update_deployment_instance(self, *args, **kwargs)

    def update_deployment_template(self, *args, **kwargs):
        """
        This method mocks the original api DeploymentsApi.update_deployment_template with MagicMock.
        """
        return self.mock_update_deployment_template(self, *args, **kwargs)




from .. import pop_utils


def test_pop_utils_remove():
    assert pop_utils.remove_pop_from_str(None, "") == ""
    assert pop_utils.remove_pop_from_str("", "") == ""
    assert pop_utils.remove_pop_from_str(" ", "") == ""
    assert pop_utils.remove_pop_from_str("foobar", "") == "foobar"
    assert pop_utils.remove_pop_from_str("foobar", "foobar") == ""
    assert pop_utils.remove_pop_from_str("foobar,1,3", "foobar") == "1,3"
    assert pop_utils.remove_pop_from_str("foobar,1,3", "foobar") == "1,3"
    assert pop_utils.remove_pop_from_str("foobar,1,3", "3") == "1,foobar"
    assert pop_utils.remove_pop_from_str("3,2,1", "") == "1,2,3"
    assert pop_utils.remove_pop_from_str("3,2,1", "3") == "1,2"


def test_pop_utils_add():
    assert pop_utils.add_pop_to_str(None, "") == ""
    assert pop_utils.add_pop_to_str("", "") == ""
    assert pop_utils.add_pop_to_str("1,2,3", "") == "1,2,3"
    assert pop_utils.add_pop_to_str("1,2,3", "") == "1,2,3"
    assert pop_utils.add_pop_to_str("1,3", "2") == "1,2,3"
    assert pop_utils.add_pop_to_str("4,3,2,1", "2") == "1,2,3,4"



from .. import context
from datetime import date
import json
from ..input_helpers import get_org_from_input_or_ctx, strip_none
import agilicus
import pem
from cryptography.hazmat.backends import default_backend
from cryptography import x509
from cryptography.hazmat.primitives import serialization
import httpx
from ..output.table import (
    spec_column,
    format_table,
    metadata_column,
    status_column,
    subtable,
    column,
)


def add_certificate(ctx, certificate=None, certificate_url=None, **kwargs):
    results = []
    if certificate:
        for certificate in pem.parse_file(certificate):
            results.append(_add_certificate_pem(ctx, str(certificate), **kwargs))
    if certificate_url:
        response = httpx.get(certificate_url)
        if response.status_code != 200:
            return results
        crt_data = response.content
        cert = x509.load_der_x509_certificate(crt_data, default_backend())
        pem_data = cert.public_bytes(serialization.Encoding.PEM).decode()
        results.append(_add_certificate_pem(ctx, str(pem_data), **kwargs))
    return results


def _add_certificate_pem(ctx, certificate, labels, **kwargs):
    token = context.get_token(ctx)
    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    apiclient = context.get_apiclient(ctx, token)
    spec = agilicus.TrustedCertificateSpec(str(certificate), **kwargs)
    if labels:
        spec.labels = [agilicus.TrustedCertificateLabelName(label) for label in labels]
    req = agilicus.TrustedCertificate(spec=spec)

    try:
        return apiclient.trusted_certs_api.create_trusted_cert(req)
    except Exception as exc:
        if exc.status == 409:
            body = exc.body
            if not body:
                raise
            result = json.loads(body)
            guid = agilicus.find_guid(result)
            req = apiclient.trusted_certs_api.get_trusted_cert(guid, org_id=org_id)
            req.spec.certificate = str(certificate)
            if not req.spec.labels:
                req.spec.labels = []
            for label in labels:
                req.spec.labels.append(agilicus.TrustedCertificateLabelName(label))
            updated_cert = apiclient.trusted_certs_api.replace_trusted_cert(
                req.metadata.id,
                req,
            )
            return updated_cert
        raise


def globalsize_get_cert_url(common_name):
    url = ""
    mappings = {"GlobalSign": "gs"}
    if "GlobalSign" not in common_name:
        return None

    for part in common_name.split(" "):
        url += mappings.get(part, part.lower())
    return f"https://secure.globalsign.com/cacert/{url}.crt"


def get_globalsign_urls(**kwargs):
    base = "https://secure.globalsign.com/cacert/gsatlasr3dvtlsca"
    year = date.today().year
    urls = []
    for year in [year, year + 1]:
        url = base + str(year)

        for q in range(1, 5):
            urls.append(url + f"q{str(q)}.crt")
    return urls


def update_globalsign_certs(ctx, **kwargs):
    results = []
    for url in get_globalsign_urls():
        results.extend(add_certificate(ctx, certificate_url=url, labels=["all"]))
    return results


def update_trusted_certificates(ctx, common_name=None, globalsign=None, **kwargs):
    results = []
    if globalsign:
        results.extend(update_globalsign_certs(ctx, **kwargs))
    if common_name:
        url = globalsize_get_cert_url(common_name)
        if not url:
            return "cannot find cert url from CN"
        return add_certificate(ctx, certificate_url=url, labels=["all"], **kwargs)
    return results


def get_certificate(ctx, certificate_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.get_trusted_cert(
        certificate_id,
        **kwargs,
    )


def delete_certificate(ctx, certificate_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.delete_trusted_cert(
        certificate_id,
        **kwargs,
    )


def add_label(ctx, label, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    labelName = agilicus.TrustedCertificateLabelName(label)
    spec = agilicus.TrustedCertificateLabelSpec(labelName, **kwargs)
    req = agilicus.TrustedCertificateLabel(spec=spec)
    return apiclient.trusted_certs_api.create_label(req)


def add_bundle(ctx, bundle, label=None, label_org_id=None, exclude=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id
    if label_org_id is None:
        label_org_id = org_id
    bundle_label = None
    if label is not None:
        label_name = agilicus.TrustedCertificateLabelName(label)
        label_spec = agilicus.TrustedCertificateLabelSpec(
            name=label_name, org_id=label_org_id
        )
        bundle_label = agilicus.TrustedCertificateBundleLabel(label=label_spec)
        if exclude is not None:
            bundle_label.exclude = exclude

    bundle_name = agilicus.TrustedCertificateBundleName(bundle)
    spec = agilicus.TrustedCertificateBundleSpec(bundle_name, **kwargs)
    if bundle_label:
        spec.labels = []
        spec.labels.append(bundle_label)
    req = agilicus.TrustedCertificateBundle(spec=spec)
    return apiclient.trusted_certs_api.create_bundle(req)


def add_bundle_label(ctx, bundle_id, label, label_org_id=None, exclude=None, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id
    if label_org_id is None:
        label_org_id = org_id

    bundle = get_bundle(ctx, bundle_id, **kwargs)
    bundle_label = None
    label_name = agilicus.TrustedCertificateLabelName(label)
    label_spec = agilicus.TrustedCertificateLabelSpec(
        name=label_name, org_id=label_org_id
    )
    bundle_label = agilicus.TrustedCertificateBundleLabel(label=label_spec)
    if exclude is not None:
        bundle_label.exclude = exclude

    if not bundle.spec.labels:
        bundle.spec.labels = []

    bundle.spec.labels.append(bundle_label)
    return apiclient.trusted_certs_api.replace_bundle(bundle_id, bundle)


def delete_bundle(ctx, bundle_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id
    return apiclient.trusted_certs_api.delete_bundle(bundle_id, **kwargs)


def list_bundles(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.list_bundles(
        **kwargs,
    ).trusted_cert_bundles


def get_bundle(ctx, bundle_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.get_bundle(
        bundle_id,
        **kwargs,
    )


def format_bundles(ctx, bundles):
    label_columns = [
        column("label"),
    ]
    columns = [
        metadata_column("id"),
        metadata_column("created"),
        spec_column("org_id"),
        spec_column("name"),
        subtable(ctx, "spec.labels", label_columns),
    ]
    return format_table(ctx, bundles, columns)


def list_labels(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.list_labels(
        **kwargs,
    ).trusted_cert_labels


def list_certificates(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs.pop("org_id", None)

    kwargs = strip_none(kwargs)
    query_results = apiclient.trusted_certs_api.list_trusted_certs(
        org_id=org_id, **kwargs
    ).trusted_certs
    return query_results


def format_labels(ctx, labels):
    columns = [
        metadata_column("created"),
        spec_column("org_id"),
        spec_column("name"),
    ]
    return format_table(ctx, labels, columns)


def format_certificates_as_text(ctx, certificates):
    columns = [
        metadata_column("id"),
        metadata_column("created"),
        spec_column("org_id"),
        spec_column("root"),
        spec_column("labels"),
        status_column("key_usage_extension"),
        status_column("subject"),
    ]
    return format_table(ctx, certificates, columns)


def list_orgs(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs["org_id"] = org_id

    kwargs = strip_none(kwargs)
    return apiclient.trusted_certs_api.list_cert_orgs(
        **kwargs,
    ).orgs


def format_orgs(ctx, orgs):
    columns = [
        column("id"),
        column("organisation"),
    ]
    return format_table(ctx, orgs, columns)



from . import context
from agilicus.agilicus_api import (
    FileShareService,
    FileShareServiceSpec,
    K8sSlug,
)

from .input_helpers import build_updated_model_validate, strip_none
from .input_helpers import update_org_from_input_or_ctx
from .output.table import (
    spec_column,
    status_column,
    format_table,
    metadata_column,
)
from .pagination import normalize_page_args
from .resource_helpers import map_resource_published, standard_page_fields

page_fields = standard_page_fields


def list_file_share_services(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    kwargs = strip_none(kwargs)
    kwargs = normalize_page_args(kwargs)
    query_results = apiclient.app_services_api.list_file_share_services(**kwargs)
    return query_results.file_share_services


def set_client_config_from_kwargs(kwargs):
    if (
        kwargs.get("windows_drive") is not None
        or kwargs.get("linux_path") is not None
        or kwargs.get("mac_path") is not None
    ):

        client_config = {
            "rules": {},
            "mount": {},
        }
        kwargs["client_config"] = [client_config]

        if kwargs["tags"] is not None:
            client_config["rules"]["tags"] = kwargs["tags"]
        kwargs.pop("tags")

        if kwargs["windows_drive"] is not None:
            client_config["mount"]["windows_config"] = {
                "name": kwargs["windows_drive"],
                "type": "mapped_drive",
            }
        kwargs.pop("windows_drive")

        if kwargs["linux_path"] is not None:
            client_config["mount"]["linux_config"] = {"path": kwargs["linux_path"]}
        kwargs.pop("linux_path")

        if kwargs["mac_path"] is not None:
            client_config["mount"]["mac_config"] = {"path": kwargs["mac_path"]}
        kwargs.pop("mac_path")


def add_file_share_service(ctx, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    kwargs = strip_none(kwargs)

    set_client_config_from_kwargs(kwargs)

    spec = FileShareServiceSpec(**kwargs)
    model = FileShareService(spec=spec)

    return apiclient.app_services_api.create_file_share_service(model).to_dict()


def _get_file_share_service(ctx, apiclient, file_share_service_id, **kwargs):
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    kwargs = strip_none(kwargs)
    return apiclient.app_services_api.get_file_share_service(
        file_share_service_id, **kwargs
    )


def show_file_share_service(ctx, file_share_service_id, **kwargs):
    kwargs = strip_none(kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    return _get_file_share_service(
        ctx, apiclient, file_share_service_id, **kwargs
    ).to_dict()


def delete_file_share_service(ctx, file_share_service_id, **kwargs):
    kwargs = strip_none(kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    return apiclient.app_services_api.delete_file_share_service(
        file_share_service_id, **kwargs
    )


def update_file_share_service(
    ctx, file_share_service_id, published, name_slug=None, **kwargs
):
    kwargs = strip_none(kwargs)
    apiclient = context.get_apiclient_from_ctx(ctx)
    get_args = {}
    update_org_from_input_or_ctx(get_args, ctx, **kwargs)
    mapping = _get_file_share_service(ctx, apiclient, file_share_service_id, **get_args)

    if name_slug is not None:
        kwargs["name_slug"] = K8sSlug(name_slug)

    old_config = mapping.spec.client_config
    mapping.spec = build_updated_model_validate(
        FileShareServiceSpec, mapping.spec, kwargs
    )
    # The updated model function messes up the config. Just replace it with the old one.
    mapping.spec.client_config = old_config
    mapping = map_resource_published(mapping, published)
    return apiclient.app_services_api.replace_file_share_service(
        file_share_service_id, file_share_service=mapping
    ).to_dict()


def format_file_share_services_as_text(ctx, shares):
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("name"),
        spec_column("share_name"),
        spec_column("local_path"),
        spec_column("connector_id"),
        status_column(in_name=["per_host_share_uri", "share_uri"], out_name="share_uri"),
    ]

    return format_table(ctx, shares, columns)



def normalize_page_args(kwargs):
    page_sort = kwargs.get("page_sort")
    if isinstance(page_sort, tuple):
        kwargs["page_sort"] = list(page_sort)
    page_on = kwargs.get("page_on")
    if isinstance(page_on, tuple):
        kwargs["page_on"] = list(page_on)
    return kwargs



import click
from agilicus.output.table import make_columns
from agilicus.output.table import output_entry
from agilicus.output.table import format_table
from agilicus.command_helpers import Command
from . import deployments


cmd = Command()


@cmd.command(name="list-deployments")
@click.option("--limit", default=500)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_deployments(ctx, **kwargs):
    results = deployments.list_deployments(ctx, **kwargs)
    columns = make_columns(
        ctx,
        results,
        """
          - metadata.id(newname=id)
          - spec.name(newname=name)
          - status.parameters:
            - name
            - type
            - default
          - status.schema_errors
        """,
    )
    print(format_table(ctx, results, columns))


@cmd.command(name="get-deployment")
@click.argument("deployment-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_get_deployment(ctx, **kwargs):
    result = deployments.get_deployment(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@cmd.command(name="delete-deployment")
@click.argument("deployment-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_deployment(ctx, **kwargs):
    deployments.delete_deployment(ctx, **kwargs)


@cmd.command(name="add-deployment")
@click.argument("name")
@click.option("--schema", type=click.Path(exists=True))
@click.option("--schema-name", default=None)
@click.option("--org-id", default=None)
@click.option("--description", default=None)
@click.pass_context
def cli_command_add_deployment(ctx, **kwargs):
    result = deployments.add_deployment(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@cmd.command(name="list-deployment-instances")
@click.option("--limit", default=500)
@click.option("--org-id", default=None)
@click.option("--show-columns", type=str, default=None)
@click.option("--reset-columns", is_flag=True, default=False)
@click.pass_context
def cli_command_list_deployment_instances(ctx, show_columns, reset_columns, **kwargs):
    results = deployments.list_deployment_instances(ctx, **kwargs)
    columns = make_columns(
        ctx,
        results,
        """
          - metadata.id(newname=id)
          - spec.name(newname=name)
          - status.status
          - status.resources:
            - template_name
            - name
            - id
            - type
            - shared
        """,
        show=show_columns,
        clear=reset_columns,
    )
    print(format_table(ctx, results, columns))


@cmd.command(name="get-deployment-instance")
@click.argument("deployment-instance-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_get_deployment_instance(ctx, **kwargs):
    result = deployments.get_deployment_instance(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@cmd.command(name="delete-deployment-instance")
@click.argument("deployment-instance-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_deployment_instance(ctx, **kwargs):
    deployments.delete_deployment_instance(ctx, **kwargs)


@cmd.command(name="list-deployment-templates")
@click.option("--limit", default=500)
@click.option("--org-id", default=None)
@click.option("--system-templates", default=True)
@click.option(
    "--deployment-template-type", type=click.Choice(["model", "schema"]), default=None
)
@click.pass_context
def cli_command_list_deployment_templates(ctx, **kwargs):
    results = deployments.list_deployment_templates(ctx, **kwargs)
    columns = make_columns(
        ctx,
        results,
        """
          - metadata.id(newname=id)
          - spec.org_id(newname=org_id)
          - spec.name(newname=name)
          - spec.template_type(newname=template_type)
          - spec.description(newname=description)
        """,
    )
    print(format_table(ctx, results, columns))


@cmd.command(name="add-deployment-instance")
@click.argument("deployment-id")
@click.argument("name")
@click.option("--org-id", default=None)
@click.option("--description", default=None)
@click.option("--input", multiple=True)
@click.pass_context
def cli_command_add_deployment_instance(ctx, **kwargs):
    result = deployments.add_deployment_instance(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@cmd.command(name="get-deployment-template")
@click.argument("deployment-template-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_get_deployment_template(ctx, **kwargs):
    result = deployments.get_deployment_template(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@cmd.command(name="add-deployment-template")
@click.argument("name")
@click.argument("template-type", type=click.Choice(["model", "schema"]))
@click.argument("template", type=click.Path(exists=True))
@click.option("--org-id", default=None)
@click.option("--description", default=None)
@click.pass_context
def cli_command_add_deployment_template(ctx, **kwargs):
    result = deployments.add_deployment_template(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@cmd.command(name="update-system-templates")
@click.argument("templates", type=click.Path(exists=True))
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_update_system_templates(ctx, **kwargs):
    if not ctx.obj.get("ADMIN_MODE"):
        print("must run in --admin mode")
        return
    deployments.update_system_templates(ctx, **kwargs)


@cmd.command(name="delete-deployment-template")
@click.argument("deployment-template-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_deployment_template(ctx, **kwargs):
    deployments.delete_deployment_template(ctx, **kwargs)


def add_commands(cli):
    cmd.add_to_cli(cli)



"""
    Agilicus API

    Agilicus is API-first. Modern software is controlled by other software, is open, is available for you to use the way you want, securely, simply.  The OpenAPI Specification in YAML format is available on [www](https://www.agilicus.com/www/api/agilicus-openapi.yaml) for importing to other tools.  A rendered, online viewable and usable version of this specification is available at [api](https://www.agilicus.com/api). You may try the API inline directly in the web page. To do so, first obtain an Authentication Token (the simplest way is to install the Python SDK, and then run `agilicus-cli --issuer https://MYISSUER get-token`). You will need an org-id for most calls (and can obtain from `agilicus-cli --issuer https://MYISSUER list-orgs`). The `MYISSUER` will typically be `auth.MYDOMAIN`, and you will see it as you sign-in to the administrative UI.  This API releases on Bearer-Token authentication. To obtain a valid bearer token you will need to Authenticate to an Issuer with OpenID Connect (a superset of OAUTH2).  Your \"issuer\" will look like https://auth.MYDOMAIN. For example, when you signed-up, if you said \"use my own domain name\" and assigned a CNAME of cloud.example.com, then your issuer would be https://auth.cloud.example.com.  If you selected \"use an Agilicus supplied domain name\", your issuer would look like https://auth.myorg.agilicus.cloud.  For test purposes you can use our [Python SDK](https://pypi.org/project/agilicus/) and run `agilicus-cli --issuer https://auth.MYDOMAIN get-token`.  This API may be used in any language runtime that supports OpenAPI 3.0, or, you may use our [Python SDK](https://pypi.org/project/agilicus/), our [Typescript SDK](https://www.npmjs.com/package/@agilicus/angular), or our [Golang SDK](https://git.agilicus.com/pub/sdk-go).  100% of the activities in our system our API-driven, from our web-admin, through our progressive web applications, to all internals: there is nothing that is not accessible.  For more information, see [developer resources](https://www.agilicus.com/developer).   # noqa: E501

    The version of the OpenAPI document: 2026.03.25
    Contact: dev@agilicus.com
    Generated by: https://openapi-generator.tech
"""


import unittest

import agilicus_api
from agilicus_api.api.applications_api import ApplicationsApi  # noqa: E501


class TestApplicationsApi(unittest.TestCase):
    """ApplicationsApi unit test stubs"""

    def setUp(self):
        self.api = ApplicationsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_add_config(self):
        """Test case for add_config

        Add an environment configuration row  # noqa: E501
        """
        pass

    def test_add_role(self):
        """Test case for add_role

        Add a role to the application.  # noqa: E501
        """
        pass

    def test_add_role_to_rule_entry(self):
        """Test case for add_role_to_rule_entry

        Add a rule to a role in the application.  # noqa: E501
        """
        pass

    def test_add_rule(self):
        """Test case for add_rule

        Add a rule to the application.  # noqa: E501
        """
        pass

    def test_create_application(self):
        """Test case for create_application

        Create an application  # noqa: E501
        """
        pass

    def test_delete_application(self):
        """Test case for delete_application

        Remove an application  # noqa: E501
        """
        pass

    def test_delete_config(self):
        """Test case for delete_config

        Remove an environment configuration  # noqa: E501
        """
        pass

    def test_delete_role(self):
        """Test case for delete_role

        Remove a role  # noqa: E501
        """
        pass

    def test_delete_role_to_rule_entry(self):
        """Test case for delete_role_to_rule_entry

        Remove a role_to_rule_entry  # noqa: E501
        """
        pass

    def test_delete_rule(self):
        """Test case for delete_rule

        Remove a rule  # noqa: E501
        """
        pass

    def test_get_all_usage_metrics(self):
        """Test case for get_all_usage_metrics

        Get all resource metrics for the Applications API  # noqa: E501
        """
        pass

    def test_get_application(self):
        """Test case for get_application

        Get a application  # noqa: E501
        """
        pass

    def test_get_application_usage_metrics(self):
        """Test case for get_application_usage_metrics

        Get application metrics  # noqa: E501
        """
        pass

    def test_get_config(self):
        """Test case for get_config

        Get environment configuration  # noqa: E501
        """
        pass

    def test_get_environment(self):
        """Test case for get_environment

        Get an environment  # noqa: E501
        """
        pass

    def test_get_role(self):
        """Test case for get_role

        Get a role  # noqa: E501
        """
        pass

    def test_get_role_to_rule_entry(self):
        """Test case for get_role_to_rule_entry

        Get a role_to_rule_entry  # noqa: E501
        """
        pass

    def test_get_rule(self):
        """Test case for get_rule

        Get a rule  # noqa: E501
        """
        pass

    def test_list_application_summaries(self):
        """Test case for list_application_summaries

        List application summaries  # noqa: E501
        """
        pass

    def test_list_applications(self):
        """Test case for list_applications

        Get applications  # noqa: E501
        """
        pass

    def test_list_combined_rules(self):
        """Test case for list_combined_rules

        List rules combined by scope or role  # noqa: E501
        """
        pass

    def test_list_configs(self):
        """Test case for list_configs

        Get all environment configuration  # noqa: E501
        """
        pass

    def test_list_environment_configs_all_apps(self):
        """Test case for list_environment_configs_all_apps

        Get all environment configuration for a given organisation.  # noqa: E501
        """
        pass

    def test_list_role_to_rule_entries(self):
        """Test case for list_role_to_rule_entries

        Get all RoleToRuleEntries  # noqa: E501
        """
        pass

    def test_list_roles(self):
        """Test case for list_roles

        Get all roles  # noqa: E501
        """
        pass

    def test_list_rules(self):
        """Test case for list_rules

        Get all rules  # noqa: E501
        """
        pass

    def test_list_runtime_status(self):
        """Test case for list_runtime_status

        Get an environment's runtime status  # noqa: E501
        """
        pass

    def test_replace_application(self):
        """Test case for replace_application

        Create or update an application  # noqa: E501
        """
        pass

    def test_replace_config(self):
        """Test case for replace_config

        Update environment configuration  # noqa: E501
        """
        pass

    def test_replace_environment(self):
        """Test case for replace_environment

        Update an environment  # noqa: E501
        """
        pass

    def test_replace_role(self):
        """Test case for replace_role

        Update a role  # noqa: E501
        """
        pass

    def test_replace_role_to_rule_entry(self):
        """Test case for replace_role_to_rule_entry

        Update a role_to_rule_entry  # noqa: E501
        """
        pass

    def test_replace_rule(self):
        """Test case for replace_rule

        Update a rule  # noqa: E501
        """
        pass

    def test_replace_runtime_status(self):
        """Test case for replace_runtime_status

        update an environemnt's runtime status  # noqa: E501
        """
        pass

    def test_update_patch_application(self):
        """Test case for update_patch_application

        patch application  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from .. import context
from ..input_helpers import (
    build_updated_model_from_dict,
    get_org_from_input_or_ctx,
    strip_none,
)
import agilicus
from ..output.table import (
    column,
    spec_column,
    format_table,
    metadata_column,
    status_column,
    constant_if_exists,
    summarize,
)

from .oauth2 import get_oauth2_auth


def make_secrets(private_key=None, **kwargs):
    private_key_data = None
    if private_key is not None:
        private_key_data = private_key.read()
    kwargs = strip_none(kwargs)

    if private_key_data is None and len(kwargs) == 0:
        # Nothing to do. Just return.
        return None

    result = agilicus.ObjectCredentialSecrets(**kwargs)
    if private_key_data is not None:
        result.private_key = private_key_data
    return result


def add_object_credentials(ctx, object_id, object_type, purpose, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    oauth2_auth = get_oauth2_auth(kwargs, pop=True)

    spec = agilicus.ObjectCredentialSpec(
        object_id=object_id,
        object_type=agilicus.ObjectType(object_type),
        purpose=agilicus.CredentialPurpose(purpose),
        **kwargs,
    )

    if oauth2_auth:
        spec.oauth2 = oauth2_auth

    creds = agilicus.ObjectCredential(spec=spec)
    return apiclient.credentials_api.create_object_credential(creds).to_dict()


def replace_object_credentials(ctx, object_credential_id, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs = strip_none(kwargs)

    org_id = get_org_from_input_or_ctx(ctx, **kwargs)
    credential = apiclient.credentials_api.get_object_credential(
        object_credential_id, org_id=org_id
    )
    kwargs = strip_none(kwargs)
    oauth2 = get_oauth2_auth(kwargs, pop=True)
    credential.spec = build_updated_model_from_dict(
        agilicus.ObjectCredentialSpec, credential.spec, kwargs
    )
    if oauth2:
        credential.spec.oauth2 = oauth2

    return apiclient.credentials_api.replace_object_credential(
        object_credential_id, object_credential=credential
    ).to_dict()


def get_object_credential(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.credentials_api.get_object_credential(**kwargs)


def delete_object_credential(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.credentials_api.delete_object_credential(**kwargs)


def list_object_credentials(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.credentials_api.list_object_credentials(**kwargs).object_credentials


def format_object_credentials(ctx, labels):
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("object_type"),
        spec_column("object_id"),
        spec_column("purpose"),
        spec_column("description"),
        status_column("is_encrypted", "enc"),
        summarize(status_column("encryption_key_id"), max_length=16),
        constant_if_exists(status_column("password"), "✓"),
        constant_if_exists(status_column("private_key", "pk"), "✓"),
        constant_if_exists(
            status_column("private_key_passphrase", "pk_passphrase"), "✓"
        ),
    ]

    return format_table(ctx, labels, columns)


def list_existence_info(ctx, **kwargs):
    token = context.get_token(ctx)
    apiclient = context.get_apiclient(ctx, token)
    kwargs["org_id"] = get_org_from_input_or_ctx(ctx, **kwargs)
    kwargs = strip_none(kwargs)

    return apiclient.credentials_api.list_object_credential_existence_info(
        **kwargs
    ).object_credential_existence_info


def format_object_credential_existence_info(ctx, labels):
    columns = [
        column("credential_id"),
        column("org_id"),
        column("object_type"),
        column("object_id"),
        column("purpose"),
    ]

    return format_table(ctx, labels, columns)



from . import context
from agilicus.agilicus_api import (
    SSHResource,
    SSHResourceSpec,
    NetworkPort,
    NetworkPortRange,
    NetworkServiceConfig,
)

from .input_helpers import build_updated_model
from .input_helpers import update_org_from_input_or_ctx
from .input_helpers import strip_none
from .output.table import (
    spec_column,
    format_table,
    metadata_column,
)
from .pagination import normalize_page_args
from .resource_helpers import map_resource_published, standard_page_fields

page_fields = standard_page_fields


def list_ssh_resources(ctx, get_all=False, limit=None, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    params = strip_none(kwargs)
    params = normalize_page_args(params)
    if not get_all:
        if limit is None:
            limit = 500
        return apiclient.app_services_api.list_ssh_resources(
            limit=limit, **params
        ).ssh_resources
    return [
        x
        for x in apiclient.app_services_api.list_ssh_resources.auto_paging_iter(
            limit=limit, **params
        )
    ]


def add_ssh_resource(ctx, port, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    spec = SSHResourceSpec(**strip_none(kwargs))
    if port is not None:
        ports = NetworkPortRange(port=NetworkPort(str(port)))
        spec["config"] = NetworkServiceConfig(ports=[ports])
    model = SSHResource(spec=spec)

    return apiclient.app_services_api.create_ssh_resource(model).to_dict()


def _get_ssh_resource(ctx, apiclient, resource_id, **kwargs):
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    return apiclient.app_services_api.get_ssh_resource(resource_id, **kwargs)


def show_ssh_resource(ctx, resource_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    return _get_ssh_resource(ctx, apiclient, resource_id, **kwargs).to_dict()


def delete_ssh_resource(ctx, resource_id, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    update_org_from_input_or_ctx(kwargs, ctx, **kwargs)
    return apiclient.app_services_api.delete_ssh_resource(resource_id, **kwargs)


def update_ssh_resource(ctx, resource_id, port, published, **kwargs):
    apiclient = context.get_apiclient_from_ctx(ctx)
    get_args = {}
    update_org_from_input_or_ctx(get_args, ctx, **kwargs)
    mapping = _get_ssh_resource(ctx, apiclient, resource_id, **get_args)

    # check_type=False works around nested types not deserializing correctly
    mapping.spec = build_updated_model(
        SSHResourceSpec, mapping.spec, kwargs, check_type=False
    )
    if port is not None:
        ports = NetworkPortRange(port=NetworkPort(str(port)))
        mapping.spec["config"] = NetworkServiceConfig(ports=[ports])

    mapping = map_resource_published(mapping, published)

    return apiclient.app_services_api.replace_ssh_resource(
        resource_id, ssh_resource=mapping
    ).to_dict()


def format_ssh_as_text(ctx, resources):
    columns = [
        metadata_column("id"),
        spec_column("org_id"),
        spec_column("name"),
        spec_column("address"),
        spec_column("connector_id"),
        spec_column("config"),
    ]

    return format_table(ctx, resources, columns)



import click
from ..output.table import output_entry
from . import trusted_certs


@click.command(name="add-trusted-certificate")
@click.option("--certificate", multiple=True, type=click.Path(exists=True))
@click.option("--certificate-url", multiple=True)
@click.option("--root", is_flag=True)
@click.option("--label", type=str, multiple=True)
@click.option("--org-id", default=None)
@click.option("--quiet", default=True)
@click.pass_context
def cli_command_add_trusted_certificate(
    ctx, certificate, certificate_url, label, quiet, **kwargs
):
    for cert in certificate:
        results = trusted_certs.add_certificate(
            ctx, certificate=cert, labels=list(label), **kwargs
        )
        if quiet:
            continue
        for result in results:
            output_entry(ctx, result.to_dict())
    for url in certificate_url:
        results = trusted_certs.add_certificate(
            ctx, certificate_url=url, labels=list(label), **kwargs
        )
        if quiet:
            continue
        for result in results:
            output_entry(ctx, result.to_dict())


@click.command(name="update-trusted-certificates")
@click.option("--globalsign", is_flag=True, default=False)
@click.option("--common-name", default=None)
@click.pass_context
def cli_command_update_trusted_certificates(ctx, **kwargs):
    if not ctx.obj.get("ADMIN_MODE"):
        print("must run in --admin mode")
        return
    results = trusted_certs.update_trusted_certificates(ctx, **kwargs)
    for result in results:
        output_entry(ctx, result.to_dict())


@click.command(name="show-trusted-certificate")
@click.argument("certificate-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_show_trusted_certificate(ctx, **kwargs):
    result = trusted_certs.get_certificate(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="delete-trusted-certificate")
@click.argument("certificate-id")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_trusted_certificate(ctx, **kwargs):
    trusted_certs.delete_certificate(ctx, **kwargs)


@click.command(name="add-trusted-certificate-label")
@click.argument("label", nargs=-1)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_add_trusted_certificate_label(ctx, label, **kwargs):
    for _label in label:
        result = trusted_certs.add_label(ctx, label=_label, **kwargs)
        output_entry(ctx, result.to_dict())


@click.command(name="list-trusted-certificate-labels")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_trusted_certificate_labels(ctx, **kwargs):
    results = trusted_certs.list_labels(ctx, **kwargs)
    print(trusted_certs.format_labels(ctx, results))


@click.command(name="list-trusted-certificate-orgs")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_trusted_certificate_orgs(ctx, **kwargs):
    results = trusted_certs.list_orgs(ctx, **kwargs)
    print(trusted_certs.format_orgs(ctx, results))


@click.command(name="list-trusted-certificates")
@click.option("--org-id", default=None)
@click.option("--trusted-cert-label", default=None)
@click.option("--subject-search", default=None)
@click.option("--issuer-search", default=None)
@click.option("--subject", default=None)
@click.option("--issuer", default=None)
@click.option("--skid", default=None)
@click.option("--serial-number", default=None)
@click.option("--cert-root", is_flag=True, default=None)
@click.option("--key-usage-extension-search", default=None)
@click.option("--key-usage-crl-sign", is_flag=True, default=None)
@click.option("--key-usage-key-cert-sign", is_flag=True, default=None)
@click.pass_context
def cli_command_list_trusted_certificates(ctx, **kwargs):
    results = trusted_certs.list_certificates(ctx, **kwargs)
    print(trusted_certs.format_certificates_as_text(ctx, results))


@click.command(name="add-trusted-certificate-bundle")
@click.argument("bundle")
@click.option("--org-id", default=None)
@click.option("--label", default=None)
@click.option("--label-org-id", default=None)
@click.option("--exclude", is_flag=True, default=None)
@click.pass_context
def cli_command_add_trusted_certificate_bundle(ctx, **kwargs):
    result = trusted_certs.add_bundle(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="add-trusted-certificate-bundle-label")
@click.argument("bundle-id")
@click.argument("label")
@click.option("--label-org-id", default=None)
@click.option("--exclude", is_flag=True, default=None)
@click.pass_context
def cli_command_add_trusted_certificate_bundle_label(ctx, **kwargs):
    result = trusted_certs.add_bundle_label(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


@click.command(name="delete-trusted-certificate-bundle")
@click.argument("bundle-id", nargs=-1)
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_delete_trusted_certificate_bundle(ctx, bundle_id, **kwargs):
    for _bundle_id in bundle_id:
        trusted_certs.delete_bundle(ctx, bundle_id=_bundle_id, **kwargs)


@click.command(name="list-trusted-certificate-bundles")
@click.option("--org-id", default=None)
@click.pass_context
def cli_command_list_trusted_certificate_bundle(ctx, **kwargs):
    results = trusted_certs.list_bundles(ctx, **kwargs)
    print(trusted_certs.format_bundles(ctx, results))


@click.command(name="get-trusted-certificate-bundle")
@click.argument("bundle-id")
@click.option("--org-id", default=None)
@click.option("--get-certs", is_flag=True, default=None)
@click.option("--trusted-cert-bundle-etag", default=None)
@click.pass_context
def cli_command_get_trusted_certificate_bundle(ctx, **kwargs):
    result = trusted_certs.get_bundle(ctx, **kwargs)
    output_entry(ctx, result.to_dict())


all_funcs = [func for func in dir() if "cli_command_" in func]


def add_commands(cli):
    glob = globals()
    for func in all_funcs:
        cli.add_command(glob[func])
